(* Copyright (c) by DATATEC Computer u. Elektronik GmbH

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *)

(******************************************************************
**
** MyHostInfo Version 1.0
**
** Author Ch. Friedrich 14.6.2001
** Tool fr MySQL-Host connection-propertys
**
*******************************************************************)
unit MyHostInfo;

interface

uses
  SysUtils, Classes, QGraphics, QControls, QForms, QDialogs,
  QExtctrls,IniFiles;
  
type

  TCryptMode = (Kodieren, Dekodieren);
  TCompError = procedure (Sender : TObject; PrgError : string) of Object;


  THostInfo = class(TComponent)
  private
    FProfiles, FHosts, FDataBases,
    FUsers, FPasswrds:  TStringList;
    FProfName: String;
    FProfilID: Integer;
    FDatabase: String;
    FProfilNo: Integer;
    FHost: String;
    FPort: Integer;
    FUser: String;
    FPassword: String;
    FIniFileName: string;
    FOnCompError:TCompError;
    FCryptKey:Byte;

  protected
    procedure CreateError(const ErrStr: String);
    function TestHostIni:boolean;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function LoadProfileItems:integer;
    function LoadProfil(No:integer):boolean;
    function SaveProfil(No:integer):boolean;

  published
    property IniFileName: String read FIniFileName write FIniFileName;
    property Profiles: TStringList read FProfiles write FProfiles;
    property Profil: String read FProfName write FProfName;
    property ProfilID: Integer read FProfilID;
    property Host: String read FHost write FHost;
    property Database: String read FDatabase write FDatabase;
    property Port: Integer read FPort write FPort;
    property User: String read FUser write FUser;
    property Password: String read FPassword write FPassword;
    property CryptKey: Byte read FCryptKey write FCryptKey;

    property OnActionError : TCompError read FOnCompError write FOnCompError;

  end;

const
   INI_READ_ERR = 'Inifile read Error';
   INI_WRITE_ERR = 'Inifile write Error';
   INI_CREATE_ERR = 'Inifile create Error';


procedure Register;

implementation

{THostInfo}

// Simple encrypt-methode for the password - compatible with Windows-Component
Function CryptPwd(Mode:TCryptMode;ChKey:integer; Value:string):string;
var s : string[255];
    c : array[0..255] of byte absolute s;
    i,xkey : Integer;
begin
  try
      xkey:=Ord(ChKey);
      s := Value;
      if trim(s) <> '' then begin
        if Mode = Kodieren then begin
            for i := 1 To ord(s[0]) do begin
             c[i] := xkey Xor c[i];
             Result :=Result+FormatFloat('000', ord(c[i]));
            end;
        end else  begin
            s:='';
            i:=1;
            while i < Length(Value) do begin
                s:=s+Char(StrToInt(copy(Value,i,3)));
                i:=i+3;
            end;
            for i := 1 To Length(s) do s[i] := chr(xkey Xor ord(s[i]));
            Result := s;
        end;
      end;
  except
    Result :='';
  end;
end;


constructor THostInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  try
     FProfiles:= TStringList.Create;
     FHosts:= TStringList.Create;
     FDataBases:= TStringList.Create;
     FUsers:= TStringList.Create;
     FPasswrds:= TStringList.Create;
     FProfName:='New Profil';
     FProfilNo:= 1;
     FHost := 'localhost';
     FPort := 3306;
     FUser := 'root';
     FPassword := '';
     FDatabase :='mysql';
     FCryptKey := 55;
     FIniFileName:='Hosts.ini';
   except
   end;
end;


destructor THostInfo.Destroy;
begin
   try
     FProfiles.Free;
     FHosts.Free;
     FDataBases.Free;
     FUsers.Free;
     FPasswrds.Free;
   except
   end;
   inherited Destroy;
end;


procedure THostInfo.CreateError(const ErrStr: String);
begin
   if Assigned(FOnCompError)
     then FOnCompError(Self, ErrStr);
end;


function THostInfo.TestHostIni:boolean;
var
  IniFile: TIniFile;
  IniFieleSL: TStringlist;
begin
  Result:=false;
  try
    if not FileExists(FIniFileName) then begin
      IniFieleSL:=TStringlist.Create;
      IniFile := TIniFile.Create(FIniFileName+'.tmp');
      IniFile.WriteString('Host1', 'ProfName', 'New Host');
      IniFile.WriteString('Host1', 'Host', 'localhost');
      IniFile.WriteString('Host1', 'Database', 'test');
      IniFile.WriteString('Host1', 'User', 'root');
      IniFile.WriteString('Host1', 'Pwd', '');
  // Kylix has a bug. The Ini-write-methodes are bad.
      IniFile.GetStrings(IniFieleSL);
      IniFieleSL.SaveToFile(FIniFileName);
      IniFile.Free;
      IniFieleSL.free;
    end;
    Result:=true;
  except
    IniFile.Free;
    IniFieleSL.free;
    CreateError(INI_CREATE_ERR);
  end;
end;


function THostInfo.LoadProfil(No:integer):boolean;
begin
  Result:=false;
  try
    if (No > 0) and (No <= FProfiles.count) then begin
    // Check values, set defaults
        FProfilID:=No;
        FProfName:= FProfiles[No-1];
        FHost:= FHosts[No-1];
        FDatabase:=FDatabases[No-1];
        FUser:=FUsers[No-1];
        FPassword:=CryptPwd(DeKodieren,FCryptKey, FPasswrds[No-1]);
        Result:= true;
     end else FProfilID:=0;
  except
    FProfilID:=0;
    CreateError(INI_READ_ERR);
  end;
end;


// Kylix has a bug. The Ini-write-methodes are bad!!!
function THostInfo.SaveProfil(No:integer):boolean;
 var
  sHost: string;
  IniFieleSL: TStringlist;
  HostCount, SLCount,i,y:integer;

  procedure WriteProfil(HostNo:integer);
  begin
      IniFieleSL.add('['+'Host'+InttoStr(HostNo)+']');
      IniFieleSL.add('ProfName='+FProfiles[HostNo-1]);
      IniFieleSL.add('Host='+FHosts[HostNo-1]);
      IniFieleSL.add('Database='+FDatabases[HostNo-1]);
      IniFieleSL.add('User='+FUsers[HostNo-1]);
      IniFieleSL.add('Pwd='+FPasswrds[HostNo-1]);
      IniFieleSL.add('');
  end;

  procedure EditProfil;
  begin
        FProfiles[No-1]:=FProfName;
        FHosts[No-1]:= FHost;
        FDatabases[No-1]:= FDatabase;
        FUsers[No-1]:=FUser;
        FPasswrds[No-1]:=CryptPwd(Kodieren,FCryptKey,FPassword);
  end;

begin
  try
    Result:=false;
    IniFieleSL:=TStringlist.Create;
    // Check values, set defaults
    if No < 1 then No:=1;
    sHost:='Host'+InttoStr(No);
    if (FCryptKey < 1) or (FCryptKey > 255) then FCryptKey:=55;
    if trim(FIniFileName) = '' then FIniFileName:=GetCurrentDir+'/Hosts.ini';
    if trim(FProfName) = '' then FProfName:='Host-No. '+InttoStr(No);
    EditProfil;
    for i:=1 to  FProfiles.Count do WriteProfil(i);
    IniFieleSL.SaveToFile(FIniFileName);
    IniFieleSL.free;
    Result:=true;
  except
    IniFieleSL.free;
    CreateError(INI_WRITE_ERR);
  end;
end;


function THostInfo.LoadProfileItems:integer;
var i:integer;
    sHost: string;
    IniFile: TIniFile;
begin
  result:=0;
  i:=1;
  FProfiles.Clear;
  FHosts.Clear;
  FDataBases.Clear;
  FUsers.Clear;
  FPasswrds.Clear;

  if TestHostIni then begin
    IniFile := TIniFile.Create(FIniFileName);
    while true do begin
      sHost:='Host'+InttoStr(i);
      FProfName:=IniFile.ReadString(sHost, 'ProfName', '');
      if trim(FProfName) <> '' then begin
       FProfiles.add(FProfName);
       FHosts.add(IniFile.ReadString(sHost, 'Host', ''));
       FDatabases.add(IniFile.ReadString(sHost, 'Database',''));
       FUsers.add(IniFile.ReadString(sHost, 'User', ''));
//CryptPwd(DeKodieren,FCryptKey,
       FPasswrds.add(IniFile.ReadString(sHost, 'Pwd', ''));
      end else break;
      inc(i);
    end;
    result:=i;
    if i>1 then LoadProfil(1)
    else CreateError('no Items');
    IniFile.Free;
  end;
end;


procedure Register;
begin
  RegisterComponents('Data Access', [THostInfo]);
end;

end.
